@@ -1,29 +1,18 @@
-# -*- coding: utf-8 -*-
-#    Copyright (C) 2004-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Eben Kenah
-#          Aric Hagberg (hagberg@lanl.gov)
-#          Christopher Ellison
-#          Ben Edwards (bedwards@cs.unm.edu)
 """Strongly connected components."""
-import warnings as _warnings
 import networkx as nx
 from networkx.utils.decorators import not_implemented_for
 
-__all__ = ['number_strongly_connected_components',
-           'strongly_connected_components',
-           'is_strongly_connected',
-           'strongly_connected_components_recursive',
-           'kosaraju_strongly_connected_components',
-           'condensation']
+__all__ = [
+    "number_strongly_connected_components",
+    "strongly_connected_components",
+    "is_strongly_connected",
+    "strongly_connected_components_recursive",
+    "kosaraju_strongly_connected_components",
+    "condensation",
+]
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
 def strongly_connected_components(G):
     """Generate nodes in strongly connected components of graph.
 
@@ -40,7 +29,7 @@ def strongly_connected_components(G):
 
     Raises
     ------
-    NetworkXNotImplemented :
+    NetworkXNotImplemented
         If G is undirected.
 
     Examples
@@ -49,8 +38,10 @@ def strongly_connected_components(G):
 
     >>> G = nx.cycle_graph(4, create_using=nx.DiGraph())
     >>> nx.add_cycle(G, [10, 11, 12])
-    >>> [len(c) for c in sorted(nx.strongly_connected_components(G),
-    ...                         key=len, reverse=True)]
+    >>> [
+    ...     len(c)
+    ...     for c in sorted(nx.strongly_connected_components(G), key=len, reverse=True)
+    ... ]
     [4, 3]
 
     If you only want the largest component, it's more efficient to
@@ -83,7 +74,8 @@ def strongly_connected_components(G):
     lowlink = {}
     scc_found = set()
     scc_queue = []
-    i = 0     # Preorder counter
+    i = 0  # Preorder counter
+    neighbors = {v: iter(G[v]) for v in G}
     for source in G:
         if source not in scc_found:
             queue = [source]
@@ -93,7 +85,7 @@ def strongly_connected_components(G):
                     i = i + 1
                     preorder[v] = i
                 done = True
-                for w in G[v]:
+                for w in neighbors[v]:
                     if w not in preorder:
                         queue.append(w)
                         done = False
@@ -118,7 +110,7 @@ def strongly_connected_components(G):
                         scc_queue.append(v)
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
 def kosaraju_strongly_connected_components(G, source=None):
     """Generate nodes in strongly connected components of graph.
 
@@ -130,12 +122,12 @@ def kosaraju_strongly_connected_components(G, source=None):
     Returns
     -------
     comp : generator of sets
-        A genrator of sets of nodes, one for each strongly connected
+        A generator of sets of nodes, one for each strongly connected
         component of G.
 
     Raises
     ------
-    NetworkXNotImplemented:
+    NetworkXNotImplemented
         If G is undirected.
 
     Examples
@@ -144,8 +136,12 @@ def kosaraju_strongly_connected_components(G, source=None):
 
     >>> G = nx.cycle_graph(4, create_using=nx.DiGraph())
     >>> nx.add_cycle(G, [10, 11, 12])
-    >>> [len(c) for c in sorted(nx.kosaraju_strongly_connected_components(G),
-    ...                         key=len, reverse=True)]
+    >>> [
+    ...     len(c)
+    ...     for c in sorted(
+    ...         nx.kosaraju_strongly_connected_components(G), key=len, reverse=True
+    ...     )
+    ... ]
     [4, 3]
 
     If you only want the largest component, it's more efficient to
@@ -162,8 +158,7 @@ def kosaraju_strongly_connected_components(G, source=None):
     Uses Kosaraju's algorithm.
 
     """
-    with nx.utils.reversed(G):
-        post = list(nx.dfs_postorder_nodes(G, source=source))
+    post = list(nx.dfs_postorder_nodes(G.reverse(copy=False), source=source))
 
     seen = set()
     while post:
@@ -172,11 +167,11 @@ def kosaraju_strongly_connected_components(G, source=None):
             continue
         c = nx.dfs_preorder_nodes(G, r)
         new = {v for v in c if v not in seen}
-        yield new
         seen.update(new)
+        yield new
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
 def strongly_connected_components_recursive(G):
     """Generate nodes in strongly connected components of graph.
 
@@ -195,7 +190,7 @@ def strongly_connected_components_recursive(G):
 
     Raises
     ------
-    NetworkXNotImplemented :
+    NetworkXNotImplemented
         If G is undirected.
 
     Examples
@@ -204,8 +199,12 @@ def strongly_connected_components_recursive(G):
 
     >>> G = nx.cycle_graph(4, create_using=nx.DiGraph())
     >>> nx.add_cycle(G, [10, 11, 12])
-    >>> [len(c) for c in sorted(nx.strongly_connected_components_recursive(G),
-    ...                         key=len, reverse=True)]
+    >>> [
+    ...     len(c)
+    ...     for c in sorted(
+    ...         nx.strongly_connected_components_recursive(G), key=len, reverse=True
+    ...     )
+    ... ]
     [4, 3]
 
     If you only want the largest component, it's more efficient to
@@ -234,6 +233,7 @@ def strongly_connected_components_recursive(G):
        Information Processing Letters 49(1): 9-14, (1994)..
 
     """
+
     def visit(v, cnt):
         root[v] = cnt
         visited[v] = cnt
@@ -241,8 +241,7 @@ def strongly_connected_components_recursive(G):
         stack.append(v)
         for w in G[v]:
             if w not in visited:
-                for c in visit(w, cnt):
-                    yield c
+                yield from visit(w, cnt)
             if w not in component:
                 root[v] = min(root[v], root[w])
         if root[v] == visited[v]:
@@ -262,11 +261,10 @@ def strongly_connected_components_recursive(G):
     stack = []
     for source in G:
         if source not in visited:
-            for c in visit(source, cnt):
-                yield c
+            yield from visit(source, cnt)
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
 def number_strongly_connected_components(G):
     """Returns number of strongly connected components in graph.
 
@@ -282,9 +280,15 @@ def number_strongly_connected_components(G):
 
     Raises
     ------
-    NetworkXNotImplemented:
+    NetworkXNotImplemented
         If G is undirected.
 
+    Examples
+    --------
+    >>> G = nx.DiGraph([(0, 1), (1, 2), (2, 0), (2, 3), (4, 5), (3, 4), (5, 6), (6, 3), (6, 7)])
+    >>> nx.number_strongly_connected_components(G)
+    3
+
     See Also
     --------
     strongly_connected_components
@@ -298,7 +302,7 @@ def number_strongly_connected_components(G):
     return sum(1 for scc in strongly_connected_components(G))
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
 def is_strongly_connected(G):
     """Test directed graph for strong connectivity.
 
@@ -315,9 +319,18 @@ def is_strongly_connected(G):
     connected : bool
       True if the graph is strongly connected, False otherwise.
 
+    Examples
+    --------
+    >>> G = nx.DiGraph([(0, 1), (1, 2), (2, 3), (3, 0), (2, 4), (4, 2)])
+    >>> nx.is_strongly_connected(G)
+    True
+    >>> G.remove_edge(2, 3)
+    >>> nx.is_strongly_connected(G)
+    False
+
     Raises
     ------
-    NetworkXNotImplemented:
+    NetworkXNotImplemented
         If G is undirected.
 
     See Also
@@ -334,12 +347,13 @@ def is_strongly_connected(G):
     """
     if len(G) == 0:
         raise nx.NetworkXPointlessConcept(
-            """Connectivity is undefined for the null graph.""")
+            """Connectivity is undefined for the null graph."""
+        )
 
-    return len(list(strongly_connected_components(G))[0]) == len(G)
+    return len(next(strongly_connected_components(G))) == len(G)
 
 
-@not_implemented_for('undirected')
+@not_implemented_for("undirected")
 def condensation(G, scc=None):
     """Returns the condensation of G.
 
@@ -369,7 +383,7 @@ def condensation(G, scc=None):
 
     Raises
     ------
-    NetworkXNotImplemented:
+    NetworkXNotImplemented
         If G is undirected.
 
     Notes
@@ -384,7 +398,7 @@ def condensation(G, scc=None):
     members = {}
     C = nx.DiGraph()
     # Add mapping dict as graph attribute
-    C.graph['mapping'] = mapping
+    C.graph["mapping"] = mapping
     if len(G) == 0:
         return C
     for i, component in enumerate(scc):
@@ -392,8 +406,9 @@ def condensation(G, scc=None):
         mapping.update((n, i) for n in component)
     number_of_components = i + 1
     C.add_nodes_from(range(number_of_components))
-    C.add_edges_from((mapping[u], mapping[v]) for u, v in G.edges()
-                     if mapping[u] != mapping[v])
+    C.add_edges_from(
+        (mapping[u], mapping[v]) for u, v in G.edges() if mapping[u] != mapping[v]
+    )
     # Add a list of members (ie original nodes) to each node (ie scc) in C.
-    nx.set_node_attributes(C, members, 'members')
+    nx.set_node_attributes(C, members, "members")
     return C
