@@ -1,12 +1,9 @@
 import pytest
+
 import networkx
 import networkx as nx
-
-from networkx.algorithms import find_cycle
-from networkx.algorithms import minimum_cycle_basis
-
-FORWARD = nx.algorithms.edgedfs.FORWARD
-REVERSE = nx.algorithms.edgedfs.REVERSE
+from networkx.algorithms import find_cycle, minimum_cycle_basis
+from networkx.algorithms.traversal.edgedfs import FORWARD, REVERSE
 
 
 class TestCycles:
