@@ -1,18 +1,19 @@
 import pytest
 
 import networkx as nx
-from networkx.testing import assert_edges_equal, assert_nodes_equal
+from networkx.utils import edges_equal, nodes_equal
 
 # Note: SubGraph views are not tested here. They have their own testing file
 
 
-class TestReverseView(object):
+class TestReverseView:
     def setup(self):
         self.G = nx.path_graph(9, create_using=nx.DiGraph())
         self.rv = nx.reverse_view(self.G)
 
     def test_pickle(self):
         import pickle
+
         rv = self.rv
         prv = pickle.loads(pickle.dumps(rv, -1))
         assert rv._node == prv._node
@@ -31,27 +32,29 @@ class TestReverseView(object):
 
     def test_exceptions(self):
         nxg = nx.graphviews
-        pytest.raises(nx.NetworkXNotImplemented, nxg.reverse_view, nx.Graph())
+        G = nx.Graph()
+        pytest.raises(nx.NetworkXNotImplemented, nxg.reverse_view, G)
 
     def test_subclass(self):
         class MyGraph(nx.DiGraph):
             def my_method(self):
                 return "me"
+
             def to_directed_class(self):
                 return MyGraph()
 
         M = MyGraph()
         M.add_edge(1, 2)
         RM = nx.reverse_view(M)
-        print("RM class",RM.__class__)
+        print("RM class", RM.__class__)
         RMC = RM.copy()
-        print("RMC class",RMC.__class__)
+        print("RMC class", RMC.__class__)
         print(RMC.edges)
         assert RMC.has_edge(2, 1)
         assert RMC.my_method() == "me"
 
 
-class TestMultiReverseView(object):
+class TestMultiReverseView:
     def setup(self):
         self.G = nx.path_graph(9, create_using=nx.MultiDiGraph())
         self.G.add_edge(4, 5)
@@ -59,6 +62,7 @@ class TestMultiReverseView(object):
 
     def test_pickle(self):
         import pickle
+
         rv = self.rv
         prv = pickle.loads(pickle.dumps(rv, -1))
         assert rv._node == prv._node
@@ -83,7 +87,17 @@ class TestMultiReverseView(object):
         pytest.raises(nx.NetworkXNotImplemented, nxg.reverse_view, MG)
 
 
-class TestToDirected(object):
+def test_generic_multitype():
+    nxg = nx.graphviews
+    G = nx.DiGraph([(1, 2)])
+    with pytest.raises(nx.NetworkXError):
+        nxg.generic_graph_view(G, create_using=nx.MultiGraph)
+    G = nx.MultiDiGraph([(1, 2)])
+    with pytest.raises(nx.NetworkXError):
+        nxg.generic_graph_view(G, create_using=nx.DiGraph)
+
+
+class TestToDirected:
     def setup(self):
         self.G = nx.path_graph(9)
         self.dv = nx.to_directed(self.G)
@@ -97,11 +111,12 @@ class TestToDirected(object):
     def test_already_directed(self):
         dd = nx.to_directed(self.dv)
         Mdd = nx.to_directed(self.Mdv)
-        assert_edges_equal(dd.edges, self.dv.edges)
-        assert_edges_equal(Mdd.edges, self.Mdv.edges)
+        assert edges_equal(dd.edges, self.dv.edges)
+        assert edges_equal(Mdd.edges, self.Mdv.edges)
 
     def test_pickle(self):
         import pickle
+
         dv = self.dv
         pdv = pickle.loads(pickle.dumps(dv, -1))
         assert dv._node == pdv._node
@@ -121,7 +136,7 @@ class TestToDirected(object):
         assert sorted(self.dv.edges) == expected
 
 
-class TestToUndirected(object):
+class TestToUndirected:
     def setup(self):
         self.DG = nx.path_graph(9, create_using=nx.DiGraph())
         self.uv = nx.to_undirected(self.DG)
@@ -135,17 +150,18 @@ class TestToUndirected(object):
     def test_already_directed(self):
         uu = nx.to_undirected(self.uv)
         Muu = nx.to_undirected(self.Muv)
-        assert_edges_equal(uu.edges, self.uv.edges)
-        assert_edges_equal(Muu.edges, self.Muv.edges)
+        assert edges_equal(uu.edges, self.uv.edges)
+        assert edges_equal(Muu.edges, self.Muv.edges)
 
     def test_pickle(self):
         import pickle
+
         uv = self.uv
         puv = pickle.loads(pickle.dumps(uv, -1))
         assert uv._node == puv._node
         assert uv._adj == puv._adj
         assert uv.graph == puv.graph
-        assert hasattr(uv, '_graph')
+        assert hasattr(uv, "_graph")
 
     def test_contains(self):
         assert (2, 3) in self.DG.edges
@@ -158,7 +174,7 @@ class TestToUndirected(object):
         assert sorted(self.uv.edges) == expected
 
 
-class TestChainsOfViews(object):
+class TestChainsOfViews:
     @classmethod
     def setup_class(cls):
         cls.G = nx.path_graph(9)
@@ -171,18 +187,28 @@ class TestChainsOfViews(object):
         cls.MDGv = nx.to_directed(cls.MG)
         cls.Rv = cls.DG.reverse()
         cls.MRv = cls.MDG.reverse()
-        cls.graphs = [cls.G, cls.DG, cls.MG, cls.MDG,
-                       cls.Gv, cls.DGv, cls.MGv, cls.MDGv,
-                       cls.Rv, cls.MRv]
+        cls.graphs = [
+            cls.G,
+            cls.DG,
+            cls.MG,
+            cls.MDG,
+            cls.Gv,
+            cls.DGv,
+            cls.MGv,
+            cls.MDGv,
+            cls.Rv,
+            cls.MRv,
+        ]
         for G in cls.graphs:
             G.edges, G.nodes, G.degree
 
     def test_pickle(self):
         import pickle
+
         for G in self.graphs:
             H = pickle.loads(pickle.dumps(G, -1))
-            assert_edges_equal(H.edges, G.edges)
-            assert_nodes_equal(H.nodes, G.nodes)
+            assert edges_equal(H.edges, G.edges)
+            assert nodes_equal(H.nodes, G.nodes)
 
     def test_subgraph_of_subgraph(self):
         SGv = nx.subgraph(self.G, range(3, 7))
@@ -198,7 +224,7 @@ class TestChainsOfViews(object):
             assert SSG._graph is G
 
     def test_restricted_induced_subgraph_chains(self):
-        """ Test subgraph chains that both restrict and show nodes/edges.
+        """Test subgraph chains that both restrict and show nodes/edges.
 
         A restricted_view subgraph should allow induced subgraphs using
         G.subgraph that automagically without a chain (meaning the result
@@ -213,19 +239,19 @@ class TestChainsOfViews(object):
         assert RG._graph is self.G
         assert SSG._graph is self.G
         assert SG._graph is RG
-        assert_edges_equal(SG.edges, SSG.edges)
+        assert edges_equal(SG.edges, SSG.edges)
         # should be same as morphing the graph
         CG = self.G.copy()
         CG.remove_nodes_from(hide_nodes)
         CG.remove_edges_from(hide_edges)
-        assert_edges_equal(CG.edges(nodes), SSG.edges)
+        assert edges_equal(CG.edges(nodes), SSG.edges)
         CG.remove_nodes_from([0, 1, 2, 3])
-        assert_edges_equal(CG.edges, SSG.edges)
+        assert edges_equal(CG.edges, SSG.edges)
         # switch order: subgraph first, then restricted view
         SSSG = self.G.subgraph(nodes)
         RSG = nx.restricted_view(SSSG, hide_nodes, hide_edges)
         assert RSG._graph is not self.G
-        assert_edges_equal(RSG.edges, CG.edges)
+        assert edges_equal(RSG.edges, CG.edges)
 
     def test_subgraph_copy(self):
         for origG in self.graphs:
@@ -276,43 +302,44 @@ class TestChainsOfViews(object):
         SG = G.subgraph([4, 5, 6])
         CSG = SG.copy(as_view=True)
         DCSG = SG.copy(as_view=False)
-        assert hasattr(CSG, '_graph')  # is a view
-        assert not hasattr(DCSG, '_graph')  # not a view
+        assert hasattr(CSG, "_graph")  # is a view
+        assert not hasattr(DCSG, "_graph")  # not a view
 
     def test_copy_disubgraph(self):
         G = self.DG.copy()
         SG = G.subgraph([4, 5, 6])
         CSG = SG.copy(as_view=True)
         DCSG = SG.copy(as_view=False)
-        assert hasattr(CSG, '_graph')  # is a view
-        assert not hasattr(DCSG, '_graph')  # not a view
+        assert hasattr(CSG, "_graph")  # is a view
+        assert not hasattr(DCSG, "_graph")  # not a view
 
     def test_copy_multidisubgraph(self):
         G = self.MDG.copy()
         SG = G.subgraph([4, 5, 6])
         CSG = SG.copy(as_view=True)
         DCSG = SG.copy(as_view=False)
-        assert hasattr(CSG, '_graph')  # is a view
-        assert not hasattr(DCSG, '_graph')  # not a view
+        assert hasattr(CSG, "_graph")  # is a view
+        assert not hasattr(DCSG, "_graph")  # not a view
 
     def test_copy_multisubgraph(self):
         G = self.MG.copy()
         SG = G.subgraph([4, 5, 6])
         CSG = SG.copy(as_view=True)
         DCSG = SG.copy(as_view=False)
-        assert hasattr(CSG, '_graph')  # is a view
-        assert not hasattr(DCSG, '_graph')  # not a view
+        assert hasattr(CSG, "_graph")  # is a view
+        assert not hasattr(DCSG, "_graph")  # not a view
 
     def test_copy_of_view(self):
         G = nx.OrderedMultiGraph(self.MGv)
-        assert G.__class__.__name__ == 'OrderedMultiGraph'
+        assert G.__class__.__name__ == "OrderedMultiGraph"
         G = G.copy(as_view=True)
-        assert G.__class__.__name__ == 'OrderedMultiGraph'
+        assert G.__class__.__name__ == "OrderedMultiGraph"
 
     def test_subclass(self):
         class MyGraph(nx.DiGraph):
             def my_method(self):
                 return "me"
+
             def to_directed_class(self):
                 return MyGraph()
 
