@@ -1,20 +1,28 @@
-#    Copyright (C) 2011-2019 by
-#
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    Michael E. Rose <Michael.Ernst.Rose@gmail.com>
-#
-#    All rights reserved.
-#    BSD license.
 from itertools import chain, count
+
 import networkx as nx
-from networkx.utils import make_str, to_tuple
-__all__ = ['node_link_data', 'node_link_graph']
+
+__all__ = ["node_link_data", "node_link_graph"]
 
 
-_attrs = dict(source='source', target='target', name='id',
-              key='key', link='links')
+_attrs = dict(source="source", target="target", name="id", key="key", link="links")
+
+
+def _to_tuple(x):
+    """Converts lists to tuples, including nested lists.
+
+    All other non-list inputs are passed through unmodified. This function is
+    intended to be used to convert potentially nested lists from json files
+    into valid nodes.
+
+    Examples
+    --------
+    >>> _to_tuple([1, 2, [3, 4]])
+    (1, 2, (3, 4))
+    """
+    if not isinstance(x, (tuple, list)):
+        return x
+    return tuple(map(_to_tuple, x))
 
 
 def node_link_data(G, attrs=None):
@@ -50,16 +58,20 @@ def node_link_data(G, attrs=None):
     Examples
     --------
     >>> from networkx.readwrite import json_graph
-    >>> G = nx.Graph([('A', 'B')])
+    >>> G = nx.Graph([("A", "B")])
     >>> data1 = json_graph.node_link_data(G)
     >>> H = nx.gn_graph(2)
-    >>> data2 = json_graph.node_link_data(H, {'link': 'edges', 'source': 'from', 'target': 'to'})
+    >>> data2 = json_graph.node_link_data(
+    ...     H, {"link": "edges", "source": "from", "target": "to"}
+    ... )
 
     To serialize with json
 
     >>> import json
     >>> s1 = json.dumps(data1)
-    >>> s2 = json.dumps(data2, default={'link': 'edges', 'source': 'from', 'target': 'to'})
+    >>> s2 = json.dumps(
+    ...     data2, default={"link": "edges", "source": "from", "target": "to"}
+    ... )
 
     Notes
     -----
@@ -78,26 +90,30 @@ def node_link_data(G, attrs=None):
         attrs = _attrs
     else:
         attrs.update({k: v for (k, v) in _attrs.items() if k not in attrs})
-    name = attrs['name']
-    source = attrs['source']
-    target = attrs['target']
-    links = attrs['link']
+    name = attrs["name"]
+    source = attrs["source"]
+    target = attrs["target"]
+    links = attrs["link"]
     # Allow 'key' to be omitted from attrs if the graph is not a multigraph.
-    key = None if not multigraph else attrs['key']
+    key = None if not multigraph else attrs["key"]
     if len({source, target, key}) < 3:
-        raise nx.NetworkXError('Attribute names are not unique.')
-    data = {'directed': G.is_directed(), 'multigraph': multigraph, 'graph': G.graph,
-            'nodes': [dict(chain(G.nodes[n].items(), [(name, n)])) for n in G]}
+        raise nx.NetworkXError("Attribute names are not unique.")
+    data = {
+        "directed": G.is_directed(),
+        "multigraph": multigraph,
+        "graph": G.graph,
+        "nodes": [dict(chain(G.nodes[n].items(), [(name, n)])) for n in G],
+    }
     if multigraph:
         data[links] = [
-            dict(chain(d.items(),
-                       [(source, u), (target, v), (key, k)]))
-            for u, v, k, d in G.edges(keys=True, data=True)]
+            dict(chain(d.items(), [(source, u), (target, v), (key, k)]))
+            for u, v, k, d in G.edges(keys=True, data=True)
+        ]
     else:
         data[links] = [
-            dict(chain(d.items(),
-                       [(source, u), (target, v)]))
-            for u, v, d in G.edges(data=True)]
+            dict(chain(d.items(), [(source, u), (target, v)]))
+            for u, v, d in G.edges(data=True)
+        ]
     return data
 
 
@@ -131,7 +147,7 @@ def node_link_graph(data, directed=False, multigraph=True, attrs=None):
     Examples
     --------
     >>> from networkx.readwrite import json_graph
-    >>> G = nx.Graph([('A', 'B')])
+    >>> G = nx.Graph([("A", "B")])
     >>> data = json_graph.node_link_data(G)
     >>> H = json_graph.node_link_graph(data)
 
@@ -148,36 +164,38 @@ def node_link_graph(data, directed=False, multigraph=True, attrs=None):
         attrs = _attrs
     else:
         attrs.update({k: v for k, v in _attrs.items() if k not in attrs})
-    multigraph = data.get('multigraph', multigraph)
-    directed = data.get('directed', directed)
+    multigraph = data.get("multigraph", multigraph)
+    directed = data.get("directed", directed)
     if multigraph:
         graph = nx.MultiGraph()
     else:
         graph = nx.Graph()
     if directed:
         graph = graph.to_directed()
-    name = attrs['name']
-    source = attrs['source']
-    target = attrs['target']
-    links = attrs['link']
+    name = attrs["name"]
+    source = attrs["source"]
+    target = attrs["target"]
+    links = attrs["link"]
     # Allow 'key' to be omitted from attrs if the graph is not a multigraph.
-    key = None if not multigraph else attrs['key']
-    graph.graph = data.get('graph', {})
+    key = None if not multigraph else attrs["key"]
+    graph.graph = data.get("graph", {})
     c = count()
-    for d in data['nodes']:
-        node = to_tuple(d.get(name, next(c)))
-        nodedata = dict((make_str(k), v) for k, v in d.items() if k != name)
+    for d in data["nodes"]:
+        node = _to_tuple(d.get(name, next(c)))
+        nodedata = {str(k): v for k, v in d.items() if k != name}
         graph.add_node(node, **nodedata)
     for d in data[links]:
         src = tuple(d[source]) if isinstance(d[source], list) else d[source]
         tgt = tuple(d[target]) if isinstance(d[target], list) else d[target]
         if not multigraph:
-            edgedata = dict((make_str(k), v) for k, v in d.items()
-                            if k != source and k != target)
+            edgedata = {str(k): v for k, v in d.items() if k != source and k != target}
             graph.add_edge(src, tgt, **edgedata)
         else:
             ky = d.get(key, None)
-            edgedata = dict((make_str(k), v) for k, v in d.items()
-                            if k != source and k != target and k != key)
+            edgedata = {
+                str(k): v
+                for k, v in d.items()
+                if k != source and k != target and k != key
+            }
             graph.add_edge(src, tgt, ky, **edgedata)
     return graph
